@@ -47,13 +47,13 @@ token=$(openstack token issue | awk 'NR==5 {print $4}')
 
 echo $token
 
-curl -X POST http://127.0.0.1:19999/v1.0/pods -H "Content-Type: application/json" \
+curl -X POST http://127.0.0.1/tricircle/v1.0/pods -H "Content-Type: application/json" \
     -H "X-Auth-Token: $token" -d '{"pod": {"region_name":  "RegionOne"}}'
 
-curl -X POST http://127.0.0.1:19999/v1.0/pods -H "Content-Type: application/json" \
+curl -X POST http://127.0.0.1/tricircle/v1.0/pods -H "Content-Type: application/json" \
     -H "X-Auth-Token: $token" -d '{"pod": {"region_name":  "Pod1", "az_name": "az1"}}'
 
-curl -X POST http://127.0.0.1:19999/v1.0/pods -H "Content-Type: application/json" \
+curl -X POST http://127.0.0.1/tricircle/v1.0/pods -H "Content-Type: application/json" \
     -H "X-Auth-Token: $token" -d '{"pod": {"region_name":  "Pod2", "az_name": "az2"}}'
 
 echo "******************************"
