@@ -20,8 +20,8 @@ import six
 from six.moves import xrange
 import uuid
 
-from keystoneclient.auth import token_endpoint
-from keystoneclient import session
+import keystoneauth1.identity.generic as auth_identity
+from keystoneauth1 import session
 from keystoneclient.v3 import client as keystone_client
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -35,10 +35,10 @@ from tricircle.db import models
 
 client_opts = [
     cfg.StrOpt('auth_url',
-               default='http://127.0.0.1/identity/v3',
+               default='http://127.0.0.1/identity',
                help='keystone authorization url'),
     cfg.StrOpt('identity_url',
-               default='http://127.0.0.1/identity_admin/v3',
+               default='http://127.0.0.1/identity/v3',
                help='keystone service url'),
     cfg.BoolOpt('auto_refresh_endpoint',
                 default=False,
@@ -70,7 +70,9 @@ client_opts = [
                choices=['p2p', 'l2gw', 'noop'],
                help='Cross-pod VxLAN networking support mode'),
     cfg.IntOpt('max_shadow_port_bulk_size', default=100,
-               help='max bulk size to create shadow ports')
+               help='max bulk size to create shadow ports'),
+    cfg.IntOpt('max_trunk_subports_bulk_size', default=100,
+               help='max bulk size to create trunk subports')
 ]
 client_opt_group = cfg.OptGroup('client')
 cfg.CONF.register_group(client_opt_group)
@@ -92,8 +94,10 @@ def _safe_operation(operation_name):
                 try:
                     service = instance.resource_service_map[resource]
                     instance._ensure_endpoint_set(context, service)
+                    instance._ensure_token_for_admin(context)
                     return func(*args, **kwargs)
                 except exceptions.EndpointNotAvailable as e:
+                    instance._unset_endpoint(service)
                     if i == retries:
                         raise
                     if cfg.CONF.client.auto_refresh_endpoint:
@@ -211,23 +215,23 @@ class Client(object):
                                 resource))
 
     @staticmethod
-    def _get_keystone_session():
-        return resource_handle.ResourceHandle.get_keystone_session()
+    def _get_keystone_session(project_id=None):
+        return resource_handle.ResourceHandle.get_keystone_session(project_id)
 
     @staticmethod
-    def get_admin_token():
-        return Client._get_admin_token()
+    def get_admin_token(project_id=None):
+        return Client._get_admin_token(project_id)
 
     @staticmethod
-    def _get_admin_token():
-        return Client._get_keystone_session().get_token()
+    def _get_admin_token(project_id=None):
+        return Client._get_keystone_session(project_id).get_token()
 
     def _get_admin_project_id(self):
         return self._get_keystone_session().get_project_id()
 
     def _get_endpoint_from_keystone(self, cxt):
-        auth = token_endpoint.Token(cfg.CONF.client.identity_url,
-                                    cxt.auth_token)
+        auth = auth_identity.Token(cfg.CONF.client.auth_url,
+                                   cxt.auth_token, tenant_id=cxt.tenant)
         sess = session.Session(auth=auth)
         cli = keystone_client.Client(session=sess)
 
@@ -260,6 +264,10 @@ class Client(object):
                                                filters, pod, service, False)
         return conf_list
 
+    def _unset_endpoint(self, service):
+        handle = self.service_handle_map[service]
+        handle.clear_endpoint_url()
+
     def _ensure_endpoint_set(self, cxt, service):
         handle = self.service_handle_map[service]
         if not handle.is_endpoint_url_set():
@@ -292,8 +300,8 @@ class Client(object):
         :return: None
         """
         if is_internal:
-            admin_context = tricircle_context.Context()
-            admin_context.auth_token = self._get_admin_token()
+            admin_context = tricircle_context.get_admin_context()
+            self._ensure_token_for_admin(admin_context)
             endpoint_map = self._get_endpoint_from_keystone(admin_context)
         else:
             endpoint_map = self._get_endpoint_from_keystone(cxt)
@@ -362,9 +370,17 @@ class Client(object):
 
     def get_keystone_client_by_context(self, ctx):
         client_session = self._get_keystone_session()
-        return keystone_client.Client(auth_url=cfg.CONF.client.identity_url,
+        return keystone_client.Client(auth_url=cfg.CONF.client.auth_url,
                                       session=client_session)
 
+    def _ensure_token_for_admin(self, cxt):
+        if cxt.is_admin and not cxt.auth_token:
+            if cxt.tenant:
+                cxt.auth_token = self._get_admin_token(cxt.tenant)
+            else:
+                cxt.auth_token = self._get_admin_token()
+                cxt.tenant = self._get_admin_project_id()
+
     @_safe_operation('client')
     def get_native_client(self, resource, cxt):
         """Get native python client instance
@@ -375,10 +391,6 @@ class Client(object):
         :param cxt: resource type
         :return: client instance
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle._get_client(cxt)
@@ -401,10 +413,6 @@ class Client(object):
         :return: list of dict containing resources information
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         filters = filters or []
@@ -435,10 +443,6 @@ class Client(object):
         :return: a dict containing resource information
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle.handle_create(cxt, resource, *args, **kwargs)
@@ -464,10 +468,6 @@ class Client(object):
         :return: a dict containing resource information
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle.handle_update(cxt, resource, *args, **kwargs)
@@ -486,10 +486,6 @@ class Client(object):
         :return: None
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle.handle_delete(cxt, resource, resource_id)
@@ -508,10 +504,6 @@ class Client(object):
         :return: a dict containing resource information
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle.handle_get(cxt, resource, resource_id)
@@ -546,10 +538,6 @@ class Client(object):
         :return: None
         :raises: EndpointNotAvailable
         """
-        if cxt.is_admin and not cxt.auth_token:
-            cxt.auth_token = self._get_admin_token()
-            cxt.tenant = self._get_admin_project_id()
-
         service = self.resource_service_map[resource]
         handle = self.service_handle_map[service]
         return handle.handle_action(cxt, resource, action, *args, **kwargs)
