@@ -13,8 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from keystoneclient.auth.identity import v3 as auth_identity
-from keystoneclient import session
+import keystoneauth1.identity.generic as auth_identity
+from keystoneauth1 import session
 
 from neutronclient.common import exceptions as q_exceptions
 from neutronclient.neutron import client as q_client
@@ -58,23 +58,30 @@ class ResourceHandle(object):
     def is_endpoint_url_set(self):
         return self.endpoint_url is not None
 
+    def clear_endpoint_url(self):
+        self.endpoint_url = None
+
     def update_endpoint_url(self, url):
         self.endpoint_url = url
 
     @staticmethod
-    def get_keystone_session():
-        auth = auth_identity.Password(
-            auth_url=cfg.CONF.client.identity_url,
-            username=cfg.CONF.client.admin_username,
-            password=cfg.CONF.client.admin_password,
-            project_name=cfg.CONF.client.admin_tenant,
-            user_domain_name=cfg.CONF.client.admin_user_domain_name,
-            project_domain_name=cfg.CONF.client.admin_tenant_domain_name)
+    def get_keystone_session(project_id=None):
+        kwargs = {
+            'auth_url': cfg.CONF.client.auth_url,
+            'username': cfg.CONF.client.admin_username,
+            'password': cfg.CONF.client.admin_password,
+            'user_domain_name': cfg.CONF.client.admin_user_domain_name,
+            'project_domain_name': cfg.CONF.client.admin_tenant_domain_name}
+        if not project_id:
+            kwargs['project_name'] = cfg.CONF.client.admin_tenant
+        else:
+            kwargs['project_id'] = project_id
+        auth = auth_identity.Password(**kwargs)
         return session.Session(auth=auth)
 
     @staticmethod
-    def get_admin_token():
-        return ResourceHandle.get_keystone_session().get_token()
+    def get_admin_token(project_id=None):
+        return ResourceHandle.get_keystone_session(project_id).get_token()
 
 
 class NeutronResourceHandle(ResourceHandle):
@@ -86,12 +93,13 @@ class NeutronResourceHandle(ResourceHandle):
         'router': LIST | CREATE | DELETE | ACTION | GET | UPDATE,
         'security_group': LIST | CREATE | GET,
         'security_group_rule': LIST | CREATE | DELETE,
-        'floatingip': LIST | CREATE | UPDATE | DELETE}
+        'floatingip': LIST | CREATE | UPDATE | DELETE,
+        'trunk': LIST | CREATE | UPDATE | GET | DELETE | ACTION}
 
     def _get_client(self, cxt):
         token = cxt.auth_token
         if not token and cxt.is_admin:
-            token = self.get_admin_token()
+            token = self.get_admin_token(cxt.tenant)
         return q_client.Client('2.0',
                                token=token,
                                auth_url=self.auth_url,
@@ -106,7 +114,6 @@ class NeutronResourceHandle(ResourceHandle):
             return [res for res in getattr(
                 client, 'list_%s' % collection)(**search_opts)[collection]]
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
 
@@ -120,7 +127,6 @@ class NeutronResourceHandle(ResourceHandle):
             else:
                 return ret['%ss' % resource]
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
 
@@ -130,7 +136,6 @@ class NeutronResourceHandle(ResourceHandle):
             return getattr(client, 'update_%s' % resource)(
                 *args, **kwargs)[resource]
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
 
@@ -139,7 +144,6 @@ class NeutronResourceHandle(ResourceHandle):
             client = self._get_client(cxt)
             return getattr(client, 'show_%s' % resource)(resource_id)[resource]
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
         except q_exceptions.NotFound:
@@ -151,7 +155,6 @@ class NeutronResourceHandle(ResourceHandle):
             client = self._get_client(cxt)
             return getattr(client, 'delete_%s' % resource)(resource_id)
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
         except q_exceptions.NotFound:
@@ -161,10 +164,11 @@ class NeutronResourceHandle(ResourceHandle):
     def handle_action(self, cxt, resource, action, *args, **kwargs):
         try:
             client = self._get_client(cxt)
-            return getattr(client, '%s_%s' % (action, resource))(*args,
-                                                                 **kwargs)
+            func_name = '%s_%s' % (action, resource)
+            if not hasattr(client, func_name):
+                func_name = '%s_%s' % (resource, action)
+            return getattr(client, func_name)(*args, **kwargs)
         except q_exceptions.ConnectionFailed:
-            self.endpoint_url = None
             raise exceptions.EndpointNotAvailable(
                 'neutron', client.httpclient.endpoint_url)
 
