@@ -25,6 +25,7 @@ from oslo_log import log
 
 from neutron.api.v2 import attributes
 from neutron.callbacks import events
+from neutron.callbacks import exceptions as callbacks_exc
 from neutron.callbacks import registry
 from neutron.callbacks import resources
 import neutron.common.exceptions as ml2_exceptions
@@ -52,6 +53,7 @@ from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
 from neutron_lib import constants
 from neutron_lib import exceptions
+from neutron_lib.plugins import directory
 import neutronclient.common.exceptions as q_cli_exceptions
 
 from sqlalchemy import sql
@@ -238,8 +240,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
     @staticmethod
     def _fill_provider_info(from_net, to_net):
-        provider_attrs = ('provider:network_type', 'provider:segmentation_id',
-                          'provider:physical_network')
+        provider_attrs = provider_net.ATTRIBUTES
         for provider_attr in provider_attrs:
             if validators.is_attr_set(from_net.get(provider_attr)):
                 to_net[provider_attr] = from_net[provider_attr]
@@ -393,7 +394,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                 t_ctx, network_id, t_constants.RT_NETWORK)
             if mappings:
                 self.xjob_handler.update_network(
-                    t_ctx, network_id, t_constants.POD_NOT_SPECIFIED)
+                    t_ctx, net['tenant_id'], network_id,
+                    t_constants.POD_NOT_SPECIFIED)
 
             self.type_manager.extend_network_dict_provider(context, net)
             return net
@@ -530,7 +532,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             mappings = db_api.get_bottom_mappings_by_top_id(
                 t_ctx, subnet_id, t_constants.RT_SUBNET)
             if mappings:
-                self.xjob_handler.update_subnet(t_ctx, subnet_id,
+                self.xjob_handler.update_subnet(t_ctx, result['tenant_id'],
+                                                subnet_id,
                                                 t_constants.POD_NOT_SPECIFIED)
             return result
 
@@ -604,23 +607,28 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                     t_ctx, sg_id, sg_id, bottom_pod['pod_id'], t_ctx.tenant,
                     t_constants.RT_SG)
 
-    @staticmethod
-    def _create_mapping_for_vm_port(t_ctx, port_body, pod):
-        entries = [(ip['subnet_id'],
-                    t_constants.RT_SUBNET) for ip in port_body['fixed_ips']]
-        entries.append((port_body['network_id'], t_constants.RT_NETWORK))
-        entries.append((port_body['id'], t_constants.RT_PORT))
-        if port_body['security_groups']:
-            for sg_id in port_body['security_groups']:
-                entries.append((sg_id, t_constants.RT_SG))
+    def _create_mapping_for_vm_port(self, t_ctx, port_body, pod):
+        entries = self.helper.extract_resource_routing_entries(port_body)
+        self.helper.ensure_resource_mapping(t_ctx, port_body['project_id'],
+                                            pod, entries)
 
-        for resource_id, resource_type in entries:
-            if db_api.get_bottom_id_by_top_id_region_name(
-                    t_ctx, resource_id, pod['region_name'], resource_type):
-                continue
-            db_api.create_resource_mapping(
-                t_ctx, resource_id, resource_id, pod['pod_id'],
-                port_body['tenant_id'], resource_type)
+    def _process_trunk_port(self, ctx, t_ctx, port_body, pod, profile_dict):
+        trunk_plugin = directory.get_plugin('trunk')
+        trunk_details = port_body.get('trunk_details')
+        if trunk_plugin and trunk_details:
+            t_trunk_id = trunk_details['trunk_id']
+            b_trunk_id = profile_dict.get(
+                t_constants.PROFILE_LOCAL_TRUNK_ID)
+            entries = [(t_trunk_id, b_trunk_id, t_constants.RT_TRUNK)]
+            trunk_plugin.update_subports_device_id(
+                ctx, trunk_details, t_trunk_id,
+                t_constants.DEVICE_OWNER_SUBPORT)
+            t_trunk = trunk_plugin.get_trunk(ctx, t_trunk_id)
+            self.helper.ensure_resource_mapping(t_ctx, t_trunk['project_id'],
+                                                pod, entries)
+            if trunk_details['sub_ports']:
+                self.xjob_handler.sync_trunk(t_ctx, t_trunk['project_id'],
+                                             t_trunk_id, pod['pod_id'])
 
     def _trigger_router_xjob_for_vm_port(self, context, port_body, pod):
         interfaces = super(TricirclePlugin, self).get_ports(
@@ -645,8 +653,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                     # for local router, job will be triggered after router
                     # interface attachment.
                     self.xjob_handler.setup_bottom_router(
-                        admin_context, port_body['network_id'],
-                        router_id, pod['pod_id'])
+                        admin_context, router['tenant_id'],
+                        port_body['network_id'], router_id, pod['pod_id'])
                     # network will be attached to only one non-local router,
                     # so we break here
                     break
@@ -687,6 +695,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
                 # entries, trigger xjob and configure security group rules for
                 # instance port
                 self._create_mapping_for_vm_port(t_ctx, res, pod)
+                self._process_trunk_port(context, t_ctx,
+                                         res, pod, profile_dict)
                 # only trigger setup_bottom_router job
                 self._trigger_router_xjob_for_vm_port(context, res, pod)
                 self.xjob_handler.configure_security_group_rules(
@@ -695,7 +705,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             if is_vxlan_network and (
                     cfg.CONF.client.cross_pod_vxlan_mode in (
                         t_constants.NM_P2P, t_constants.NM_L2GW)):
-                self.xjob_handler.setup_shadow_ports(t_ctx, pod['pod_id'],
+                self.xjob_handler.setup_shadow_ports(t_ctx, res['tenant_id'],
+                                                     pod['pod_id'],
                                                      res['network_id'])
         # for vm port or port with empty device_owner, update top port and
         # bottom port
@@ -747,6 +758,25 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         self._log_update_port_sensitive_attrs(port_id, port)
         return res
 
+    def _pre_delete_port(self, context, port_id, port_check):
+        """Do some preliminary operations before deleting the port."""
+        LOG.debug("Deleting port %s", port_id)
+        try:
+            # notify interested parties of imminent port deletion;
+            # a failure here prevents the operation from happening
+            kwargs = {
+                'context': context,
+                'port_id': port_id,
+                'port_check': port_check
+            }
+            registry.notify(
+                resources.PORT, events.BEFORE_DELETE, self, **kwargs)
+        except callbacks_exc.CallbackFailure as e:
+            # NOTE(xiulin): preserve old check's behavior
+            if len(e.errors) == 1:
+                raise e.errors[0].error
+            raise exceptions.ServicePortInUse(port_id=port_id, reason=e)
+
     def delete_port(self, context, port_id, l3_port_check=True):
         t_ctx = t_context.get_context_from_neutron_context(context)
         port = super(TricirclePlugin, self).get_port(context, port_id)
@@ -762,14 +792,15 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         # delete the corresponding local ports. to avoid infinite api calls,
         # we use a "delete_server_port" job to delete the local ports.
         if port.get('device_owner') not in NON_VM_PORT_TYPES:
+            self._pre_delete_port(context, port_id, False)
             try:
                 # since we don't create resource routing entries for shadow
                 # ports, we traverse pods where the network is located to
                 # delete ports
                 for pod, _id in self.helper.get_real_shadow_resource_iterator(
                         t_ctx, t_constants.RT_NETWORK, port['network_id']):
-                    self.xjob_handler.delete_server_port(t_ctx, port_id,
-                                                         pod['pod_id'])
+                    self.xjob_handler.delete_server_port(
+                        t_ctx, port['tenant_id'], port_id, pod['pod_id'])
             except Exception:
                 raise
             with t_ctx.session.begin():
@@ -872,6 +903,7 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         else:
             ret.extend(self._get_ports_from_db_with_number(
                 context, number - len(ret), ret[-1]['id'], top_bottom_map))
+            return ret
 
     def _get_ports_from_top_with_number(self, context,
                                         number, last_port_id, top_bottom_map,
@@ -1003,6 +1035,13 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         # receives list request with "id" filter, it will query central
         # Neutron server and try to create the port. Here we introduce a
         # special handle for "id" filter
+
+        trunk_plugin = directory.get_plugin('trunk')
+        if trunk_plugin:
+            res = trunk_plugin.get_trunk_subports(context, filters)
+            if res is not None:
+                return res
+
         if not filters or 'id' not in filters:
             # if filter is empty or "id" is not in the filter, no special
             # handle is required
@@ -1514,6 +1553,10 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
             t_constants.RT_ROUTER if is_local_router
             else t_constants.RT_NS_ROUTER)
 
+        if not b_router_id:
+            # local router doesn't exist, skip local gateway deletion
+            return
+
         b_client = self._get_client(region_name)
         b_client.action_routers(t_ctx, 'remove_gateway', b_router_id)
 
@@ -1554,7 +1597,8 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         t_ctx = t_context.get_context_from_neutron_context(context)
         is_local_router = self.helper.is_local_router(t_ctx, router)
         if not is_local_router:
-            self.xjob_handler.configure_extra_routes(t_ctx, router_id)
+            self.xjob_handler.configure_route(
+                t_ctx, ret['tenant_id'], router_id)
         return ret
 
     def validate_router_net_location_match(self, t_ctx, router, net):
@@ -1674,10 +1718,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         try:
             if len(b_pods) == 1:
                 self.xjob_handler.setup_bottom_router(
-                    t_ctx, net_id, router_id, b_pods[0]['pod_id'])
+                    t_ctx, project_id, net_id, router_id, b_pods[0]['pod_id'])
             else:
                 self.xjob_handler.setup_bottom_router(
-                    t_ctx, net_id, router_id, t_constants.POD_NOT_SPECIFIED)
+                    t_ctx, project_id, net_id, router_id,
+                    t_constants.POD_NOT_SPECIFIED)
         except Exception:
             # NOTE(zhiyuan) we fail to submit the job, so bottom router
             # operations are not started, it's safe for us to remove the top
@@ -1703,15 +1748,20 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
 
         return_info = super(TricirclePlugin, self).remove_router_interface(
             context, router_id, interface_info)
+
+        router = self._get_router(context, router_id)
+
         if not b_pods:
             return return_info
         try:
             if len(b_pods) == 1:
                 self.xjob_handler.setup_bottom_router(
-                    t_ctx, net_id, router_id, b_pods[0]['pod_id'])
+                    t_ctx, router['tenant_id'], net_id,
+                    router_id, b_pods[0]['pod_id'])
             else:
                 self.xjob_handler.setup_bottom_router(
-                    t_ctx, net_id, router_id, t_constants.POD_NOT_SPECIFIED)
+                    t_ctx, router['tenant_id'], net_id,
+                    router_id, t_constants.POD_NOT_SPECIFIED)
         except Exception:
             # NOTE(zhiyuan) we fail to submit the job, so if bottom router
             # interface exists, it would not be deleted, then after we add
@@ -1832,8 +1882,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         int_net_pod, b_int_port_id = mappings[0]
         int_port = self.get_port(context, int_port_id)
         net_id = int_port['network_id']
+        router_id = floatingip_db['router_id']
+        router = self._get_router(context, router_id)
         self.xjob_handler.setup_bottom_router(
-            t_ctx, net_id, floatingip_db['router_id'], int_net_pod['pod_id'])
+            t_ctx, router['tenant_id'], net_id,
+            floatingip_db['router_id'], int_net_pod['pod_id'])
 
     def _disassociate_floatingip(self, context, ori_floatingip_db):
         if not ori_floatingip_db['port_id']:
@@ -1858,9 +1911,11 @@ class TricirclePlugin(db_base_plugin_v2.NeutronDbPluginV2,
         b_int_net_pod, b_int_port_id = mappings[0]
         int_port = self.get_port(context, t_int_port_id)
         net_id = int_port['network_id']
+        router_id = ori_floatingip_db['router_id']
+        router = self._get_router(context, router_id)
         self.xjob_handler.setup_bottom_router(
-            t_ctx, net_id, ori_floatingip_db['router_id'],
-            b_int_net_pod['pod_id'])
+            t_ctx, router['tenant_id'], net_id,
+            ori_floatingip_db['router_id'], b_int_net_pod['pod_id'])
 
     def delete_floatingip(self, context, _id):
         """Disassociate floating ip if needed then delete it
