@@ -283,13 +283,13 @@ class NetworkHelper(object):
                 'admin_state_up': True
             }
         }
-        network_type = network.get('provider:network_type')
+        network_type = network.get(provider_net.NETWORK_TYPE)
         if network_type == t_constants.NT_VLAN:
-            body['network']['provider:network_type'] = 'vlan'
-            body['network']['provider:physical_network'] = network[
-                'provider:physical_network']
-            body['network']['provider:segmentation_id'] = network[
-                'provider:segmentation_id']
+            body['network'][provider_net.NETWORK_TYPE] = 'vlan'
+            body['network'][provider_net.PHYSICAL_NETWORK] = network[
+                provider_net.PHYSICAL_NETWORK]
+            body['network'][provider_net.SEGMENTATION_ID] = network[
+                provider_net.SEGMENTATION_ID]
         return body
 
     @staticmethod
@@ -364,6 +364,9 @@ class NetworkHelper(object):
         """
         pools = t_subnet['allocation_pools']
         t_gateway_ip = t_subnet['gateway_ip']
+        if not t_gateway_ip:
+            # gateway is None, so we don't need to split allocation pools
+            return pools
         new_pools = NetworkHelper._split_pools_by_bottom_gateway_ip(
             pools, b_gateway_ip)
         if t_gateway_ip == b_gateway_ip:
@@ -471,7 +474,7 @@ class NetworkHelper(object):
         """
         # network
         net_body = self.get_create_network_body(project_id, t_net)
-        if net_body['network'].get('provider:network_type'):
+        if net_body['network'].get(provider_net.NETWORK_TYPE):
             # if network type specified, we need to switch to admin account
             admin_context = t_context.get_admin_context()
 
@@ -867,11 +870,12 @@ class NetworkHelper(object):
             port_body[portbindings.VIF_TYPE] = portbindings.VIF_TYPE_OVS
             port_body[portbindings.VNIC_TYPE] = portbindings.VNIC_NORMAL
 
-    def _prepare_shadow_ports_with_retry(self, ctx, client, req_create_bodys):
+    @staticmethod
+    def prepare_ports_with_retry(ctx, client, req_create_bodys):
         create_body_map = dict(
             [(create_body['mac_address'],
               create_body) for create_body in req_create_bodys])
-        max_tries = 5
+        max_tries = 10
         conflict_port_ids = []
         for i in xrange(max_tries):
             create_bodys = list(create_body_map.values())
@@ -892,8 +896,13 @@ class NetworkHelper(object):
                         # rare case, we conflicted with an unrecognized mac
                         raise
                     conflict_port = create_body_map.pop(conflict_mac)
-                    conflict_port_ids.append(
-                        conflict_port['name'].split('_')[-1])
+                    if (conflict_port['device_owner'] ==
+                            t_constants.DEVICE_OWNER_SHADOW):
+                        conflict_port_ids.append(
+                            conflict_port['name'].split('_')[-1])
+                    elif (conflict_port['device_owner'] ==
+                            t_constants.DEVICE_OWNER_SUBPORT):
+                        conflict_port_ids.append(conflict_port['device_id'])
                 else:
                     # the exception no longer contains mac information
                     raise
@@ -933,7 +942,7 @@ class NetworkHelper(object):
         ret_port_ids = []
         client = self._get_client(target_pod['region_name'])
         while cursor < len(full_create_bodys):
-            ret_port_ids.extend(self._prepare_shadow_ports_with_retry(
+            ret_port_ids.extend(self.prepare_ports_with_retry(
                 ctx, client,
                 full_create_bodys[cursor: cursor + max_bulk_size]))
             cursor += max_bulk_size
@@ -979,3 +988,38 @@ class NetworkHelper(object):
                 continue
             processed_pod_set.add(region_name)
             yield pod, bottom_res_id
+
+    @staticmethod
+    def extract_resource_routing_entries(port):
+        entries = []
+        if not port:
+            return entries
+        for ip in port['fixed_ips']:
+            entries.append((ip['subnet_id'], ip['subnet_id'],
+                            t_constants.RT_SUBNET))
+        entries.append((port['network_id'], port['network_id'],
+                        t_constants.RT_NETWORK))
+        entries.append((port['id'], port['id'],
+                        t_constants.RT_PORT))
+        if port['security_groups']:
+            for sg_id in port['security_groups']:
+                entries.append((sg_id, sg_id, t_constants.RT_SG))
+        return entries
+
+    @staticmethod
+    def ensure_resource_mapping(t_ctx, project_id, pod, entries):
+        """Ensure resource mapping
+
+        :param t_ctx: tricircle context
+        :param project_id: project id
+        :param pod: bottom pod
+        :param entries: a list of (top_id, bottom_id, resource_type) tuples.
+        :return: None
+        """
+        for top_id, btm_id, resource_type in entries:
+            if db_api.get_bottom_id_by_top_id_region_name(
+                    t_ctx, top_id, pod['region_name'], resource_type):
+                continue
+            db_api.create_resource_mapping(t_ctx, top_id, btm_id,
+                                           pod['pod_id'], project_id,
+                                           resource_type)
