@@ -23,6 +23,7 @@ from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import validators
 import neutron_lib.constants as q_constants
 import neutron_lib.exceptions as q_exceptions
+from neutron_lib.plugins import directory
 
 from neutron.common import utils
 from neutron.extensions import availability_zone as az_ext
@@ -43,6 +44,9 @@ from tricircle.network import helper
 tricircle_opts = [
     cfg.StrOpt('real_core_plugin', help=_('The core plugin the Tricircle '
                                           'local plugin will invoke.')),
+    cfg.StrOpt('local_region_name',
+               help=_('Region the local Neutron belongs to, has high priority '
+                      'than nova.region_name')),
     cfg.StrOpt('central_neutron_url', help=_('Central Neutron server url')),
     cfg.IPOpt('l2gw_tunnel_ip', help=_('Tunnel IP of L2 gateway, need to set '
                                        'when client.cross_pod_vxlan_mode is '
@@ -71,6 +75,8 @@ class TricirclePlugin(plugin.Ml2Plugin):
             cfg.CONF.client.auth_url)
         self.neutron_handle.endpoint_url = \
             cfg.CONF.tricircle.central_neutron_url
+        self.on_trunk_create = {}
+        self.on_subnet_delete = {}
 
     def start_rpc_listeners(self):
         return self.core_plugin.start_rpc_listeners()
@@ -84,6 +90,22 @@ class TricirclePlugin(plugin.Ml2Plugin):
     def rpc_state_report_workers_supported(self):
         return self.core_plugin.rpc_state_report_workers_supported()
 
+    def _start_subnet_delete(self, context):
+        if context.request_id:
+            LOG.debug('subnet delete start for ' + context.request_id)
+            self.on_subnet_delete[context.request_id] = True
+
+    def _end_subnet_delete(self, context):
+        if context.request_id:
+            LOG.debug('subnet delete end for ' + context.request_id)
+            self.on_subnet_delete.pop(context.request_id, None)
+
+    def _in_subnet_delete(self, context):
+        if context.request_id:
+            LOG.debug('check subnet delete state for ' + context.request_id)
+            return context.request_id in self.on_subnet_delete
+        return False
+
     @staticmethod
     def _adapt_network_body(network):
         network_type = network.get(provider_net.NETWORK_TYPE)
@@ -135,6 +157,13 @@ class TricirclePlugin(plugin.Ml2Plugin):
     def _skip_non_api_query(context):
         return not context.auth_token
 
+    @staticmethod
+    def _get_neutron_region():
+        region_name = cfg.CONF.tricircle.local_region_name
+        if not region_name:
+            region_name = cfg.CONF.nova.region_name
+        return region_name
+
     def _ensure_network_subnet(self, context, port):
         network_id = port['network_id']
         # get_network will create bottom network if it doesn't exist, also
@@ -183,7 +212,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
         self.core_plugin.create_port(q_ctx, dhcp_port_body)
 
     def _ensure_gateway_port(self, t_ctx, t_subnet):
-        region_name = cfg.CONF.nova.region_name
+        region_name = self._get_neutron_region()
         gateway_port_name = t_constants.interface_port_name % (region_name,
                                                                t_subnet['id'])
         gateway_port_body = {
@@ -228,8 +257,13 @@ class TricirclePlugin(plugin.Ml2Plugin):
     def get_network(self, context, _id, fields=None):
         try:
             b_network = self.core_plugin.get_network(context, _id)
-            subnet_ids = self._ensure_subnet(context, b_network, False)
+            if not self._in_subnet_delete(context):
+                subnet_ids = self._ensure_subnet(context, b_network, False)
+            else:
+                subnet_ids = []
         except q_exceptions.NotFound:
+            if self._in_subnet_delete(context):
+                raise
             t_ctx = t_context.get_context_from_neutron_context(context)
             if self._skip_non_api_query(t_ctx):
                 raise q_exceptions.NetworkNotFound(net_id=_id)
@@ -269,7 +303,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
         t_ctx = t_context.get_context_from_neutron_context(context)
         if self._skip_non_api_query(t_ctx):
             return b_networks
-        t_ctx.auth_token = client.Client.get_admin_token()
+        t_ctx.auth_token = client.Client.get_admin_token(context.project_id)
         raw_client = self.neutron_handle._get_client(t_ctx)
         params = self._construct_params(filters, sorts, limit, marker,
                                         page_reverse)
@@ -282,7 +316,7 @@ class TricirclePlugin(plugin.Ml2Plugin):
             missing_networks = [network for network in t_networks if (
                 network['id'] in missing_id_set)]
             for network in missing_networks:
-                region_name = cfg.CONF.nova.region_name
+                region_name = self._get_neutron_region()
                 located = self._is_network_located_in_region(network,
                                                              region_name)
                 if not located:
@@ -318,10 +352,14 @@ class TricirclePlugin(plugin.Ml2Plugin):
         return b_subnet
 
     def _create_bottom_subnet(self, t_ctx, q_ctx, t_subnet):
-        gateway_port = self._ensure_gateway_port(t_ctx, t_subnet)
+        if t_subnet['gateway_ip']:
+            gateway_port = self._ensure_gateway_port(t_ctx, t_subnet)
+            b_gateway_ip = gateway_port['fixed_ips'][0]['ip_address']
+        else:
+            b_gateway_ip = None
         subnet_body = helper.NetworkHelper.get_create_subnet_body(
-            gateway_port['tenant_id'], t_subnet, t_subnet['network_id'],
-            gateway_port['fixed_ips'][0]['ip_address'])['subnet']
+            t_subnet['tenant_id'], t_subnet, t_subnet['network_id'],
+            b_gateway_ip)['subnet']
         t_subnet['gateway_ip'] = subnet_body['gateway_ip']
         t_subnet['allocation_pools'] = subnet_body['allocation_pools']
 
@@ -384,6 +422,15 @@ class TricirclePlugin(plugin.Ml2Plugin):
                 b_subnets.append(self._fields(b_subnet, fields))
         return b_subnets
 
+    def delete_subnet(self, context, _id):
+        self._start_subnet_delete(context)
+        try:
+            self.core_plugin.delete_subnet(context, _id)
+        except Exception:
+            raise
+        finally:
+            self._end_subnet_delete(context)
+
     def update_subnet(self, context, _id, subnet):
         """update bottom subnet
 
@@ -436,15 +483,25 @@ class TricirclePlugin(plugin.Ml2Plugin):
 
     def create_port_bulk(self, context, ports):
         # NOTE(zhiyuan) currently this bulk operation is only for shadow port
-        # creation optimization
+        # and trunk subports creation optimization
         for port in ports['ports']:
             port_body = port['port']
-            self._create_shadow_agent(context, port_body)
             self.get_network(context, port_body['network_id'])
-            port_body['id'] = port_body['name'].split('_')[-1]
-            helper.NetworkHelper.fill_binding_info(port_body)
-            # clear binding profile set by xmanager
-            port_body[portbindings.PROFILE] = {}
+            if port_body['device_owner'] == t_constants.DEVICE_OWNER_SHADOW:
+                port_body['id'] = port_body['name'].split('_')[-1]
+                self._create_shadow_agent(context, port_body)
+                helper.NetworkHelper.fill_binding_info(port_body)
+                # clear binding profile set by xmanager
+                port_body[portbindings.PROFILE] = {}
+            elif (port_body['device_owner'] ==
+                    t_constants.DEVICE_OWNER_SUBPORT):
+                port_body['id'] = port_body['device_id']
+                # need set port's device_id to empty, otherwise will raise
+                # a exception because the device_id is bound to a device when
+                # the trunk add this port as a subport
+                port_body['device_owner'] = ''
+                port_body['device_id'] = ''
+
         return self.core_plugin.create_port_bulk(context, ports)
 
     def create_port(self, context, port):
@@ -604,23 +661,78 @@ class TricirclePlugin(plugin.Ml2Plugin):
                 portbindings.VNIC_TYPE] = q_constants.ATTR_NOT_SPECIFIED
         b_port = self.core_plugin.update_port(context, _id, port)
         if self._need_top_update(b_port, port['port']):
-            region_name = cfg.CONF.nova.region_name
+            region_name = self._get_neutron_region()
             update_dict = {portbindings.PROFILE: {
                 t_constants.PROFILE_REGION: region_name,
                 t_constants.PROFILE_DEVICE: b_port['device_owner']}}
             self._fill_agent_info_in_profile(
                 context, _id, port['port'][portbindings.HOST_ID],
                 update_dict[portbindings.PROFILE])
+
+            if directory.get_plugin('trunk'):
+                trunk_details = b_port.get('trunk_details')
+                if trunk_details:
+                    update_dict['binding:profile'].update({
+                        t_constants.PROFILE_LOCAL_TRUNK_ID:
+                            trunk_details['trunk_id']})
+
             t_ctx = t_context.get_context_from_neutron_context(context)
             self.neutron_handle.handle_update(t_ctx, 'port', _id,
                                               {'port': update_dict})
         return b_port
 
+    def _start_trunk_create(self, context):
+        if context.request_id:
+            LOG.debug('trunk create start for ' + context.request_id)
+            self.on_trunk_create[context.request_id] = True
+
+    def _end_trunk_create(self, context):
+        if context.request_id:
+            LOG.debug('trunk create end for ' + context.request_id)
+            self.on_trunk_create.pop(context.request_id, None)
+
+    def _in_trunk_create(self, context):
+        if context.request_id:
+            return context.request_id in self.on_trunk_create
+        return False
+
+    def _create_trunk(self, context, t_ctx, port_id):
+        trunk_plugin = directory.get_plugin('trunk')
+        if not trunk_plugin:
+            return
+        b_trunks = trunk_plugin.get_trunks(
+            context, filters={'port_id': [port_id]})
+        if b_trunks:
+            return
+        t_trunks = self.neutron_handle.handle_list(
+            t_ctx, 'trunk', [{'key': 'port_id',
+                              'comparator': 'eq',
+                              'value': port_id}])
+        if not t_trunks:
+            return
+        t_trunk = t_trunks[0]
+        # sub_ports will be created in xjob, so set it to empty here
+        t_trunk['sub_ports'] = []
+        trunk_plugin.create_trunk(context, {'trunk': t_trunk})
+
+    def _ensure_trunk(self, context, t_ctx, port_id):
+        # avoid recursive calls: _ensure_trunk will call create_trunk,
+        # create_trunk will call get_port, and get_port will call
+        # _ensure_trunk again
+        if not self._in_trunk_create(context):
+            self._start_trunk_create(context)
+            try:
+                self._create_trunk(context, t_ctx, port_id)
+            except Exception:
+                raise
+            finally:
+                self._end_trunk_create(context)
+
     def get_port(self, context, _id, fields=None):
+        t_ctx = t_context.get_context_from_neutron_context(context)
         try:
             b_port = self.core_plugin.get_port(context, _id, fields)
         except q_exceptions.NotFound:
-            t_ctx = t_context.get_context_from_neutron_context(context)
             if self._skip_non_api_query(t_ctx):
                 raise q_exceptions.PortNotFound(port_id=_id)
             t_port = self.neutron_handle.handle_get(t_ctx, 'port', _id)
@@ -630,6 +742,8 @@ class TricirclePlugin(plugin.Ml2Plugin):
             self._adapt_port_body_for_call(t_port)
             self._handle_security_group(t_ctx, context, t_port)
             b_port = self.core_plugin.create_port(context, {'port': t_port})
+
+        self._ensure_trunk(context, t_ctx, _id)
         return self._fields(b_port, fields)
 
     def get_ports(self, context, filters=None, fields=None, sorts=None,
