@@ -22,42 +22,44 @@ import unittest
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
+from neutron.services.trunk import exceptions as t_exc
 from neutron_lib.api.definitions import portbindings
 import neutron_lib.constants as q_constants
 import neutron_lib.exceptions as q_exceptions
+from neutron_lib.plugins import directory
 
 from tricircle.common import client
 from tricircle.common import constants
 import tricircle.common.context as t_context
 from tricircle.network import helper
 import tricircle.network.local_plugin as plugin
+import tricircle.tests.unit.utils as test_utils
 
 
-TOP_NETS = []
-TOP_SUBNETS = []
-TOP_PORTS = []
-TOP_SGS = []
-BOTTOM_NETS = []
-BOTTOM_SUBNETS = []
-BOTTOM_PORTS = []
-BOTTOM_SGS = []
-BOTTOM_AGENTS = []
-RES_LIST = [TOP_NETS, TOP_SUBNETS, TOP_PORTS, TOP_SGS,
-            BOTTOM_NETS, BOTTOM_SUBNETS, BOTTOM_PORTS, BOTTOM_SGS,
-            BOTTOM_AGENTS]
-RES_MAP = {'network': {True: TOP_NETS, False: BOTTOM_NETS},
-           'subnet': {True: TOP_SUBNETS, False: BOTTOM_SUBNETS},
-           'port': {True: TOP_PORTS, False: BOTTOM_PORTS},
-           'security_group': {True: TOP_SGS, False: BOTTOM_SGS},
-           'agent': {True: [], False: BOTTOM_AGENTS}}
+_resource_store = test_utils.get_resource_store()
+TOP_NETS = _resource_store.TOP_NETWORKS
+TOP_SUBNETS = _resource_store.TOP_SUBNETS
+TOP_PORTS = _resource_store.TOP_PORTS
+TOP_SGS = _resource_store.TOP_SECURITYGROUPS
+TOP_TRUNKS = _resource_store.TOP_TRUNKS
+BOTTOM_NETS = _resource_store.BOTTOM1_NETWORKS
+BOTTOM_SUBNETS = _resource_store.BOTTOM1_SUBNETS
+BOTTOM_PORTS = _resource_store.BOTTOM1_PORTS
+BOTTOM_SGS = _resource_store.BOTTOM1_SECURITYGROUPS
+BOTTOM_AGENTS = _resource_store.BOTTOM1_AGENTS
+
+
+def get_resource_list(_type, is_top):
+    pod = 'top' if is_top else 'pod_1'
+    return _resource_store.pod_store_map[pod][_type]
 
 
 def create_resource(_type, is_top, body):
-    RES_MAP[_type][is_top].append(body)
+    get_resource_list(_type, is_top).append(body)
 
 
 def update_resource(_type, is_top, resource_id, body):
-    for resource in RES_MAP[_type][is_top]:
+    for resource in get_resource_list(_type, is_top):
         if resource['id'] == resource_id:
             resource.update(body)
             return copy.deepcopy(resource)
@@ -65,17 +67,19 @@ def update_resource(_type, is_top, resource_id, body):
 
 
 def get_resource(_type, is_top, resource_id):
-    for resource in RES_MAP[_type][is_top]:
+    for resource in get_resource_list(_type, is_top):
         if resource['id'] == resource_id:
             return copy.deepcopy(resource)
     raise q_exceptions.NotFound()
 
 
 def list_resource(_type, is_top, filters=None):
+    resource_list = get_resource_list(_type, is_top)
     if not filters:
-        return [copy.deepcopy(resource) for resource in RES_MAP[_type][is_top]]
+        return [copy.deepcopy(resource) for resource in get_resource_list(
+            _type, is_top)]
     ret = []
-    for resource in RES_MAP[_type][is_top]:
+    for resource in resource_list:
         pick = True
         for key, value in six.iteritems(filters):
             if resource.get(key) not in value:
@@ -86,10 +90,6 @@ def list_resource(_type, is_top, filters=None):
     return ret
 
 
-def delete_resource(_type, is_top, body):
-    RES_MAP[_type][is_top].append(body)
-
-
 class FakeCorePlugin(object):
     supported_extension_aliases = ['agent']
 
@@ -143,22 +143,29 @@ class FakeCorePlugin(object):
         pass
 
 
-class FakeSession(object):
-    class WithWrapper(object):
-        def __enter__(self):
-            pass
-
-        def __exit__(self, type, value, traceback):
-            pass
-
-    def begin(self, subtransactions=True):
-        return FakeSession.WithWrapper()
-
-
 class FakeContext(object):
     def __init__(self):
-        self.session = FakeSession()
+        self.session = test_utils.FakeSession()
         self.auth_token = 'token'
+        self.project_id = ''
+        self.request_id = 'req-' + uuidutils.generate_uuid()
+
+
+def fake_get_trunk_plugin(trunk):
+    return FakeTrunkPlugin()
+
+
+class FakeTrunkPlugin(object):
+
+    def get_trunk(self, context, trunk_id, fields=None):
+        raise t_exc.TrunkNotFound(trunk_id=trunk_id)
+
+    def get_trunks(self, context, filters=None, fields=None,
+                   sorts=None, limit=None, marker=None, page_reverse=False):
+        return []
+
+    def create_trunk(self, context, trunk):
+        pass
 
 
 class FakeClient(object):
@@ -223,11 +230,20 @@ class FakeNeutronHandle(object):
     def handle_update(self, context, _type, _id, body):
         pass
 
+    def handle_list(self, cxt, resource, filters):
+        if resource == 'trunk':
+            for trunk in TOP_TRUNKS:
+                if trunk['port_id'] == filters[0]['value']:
+                    return [trunk]
+        return []
+
 
 class FakePlugin(plugin.TricirclePlugin):
     def __init__(self):
         self.core_plugin = FakeCorePlugin()
         self.neutron_handle = FakeNeutronHandle()
+        self.on_trunk_create = {}
+        self.on_subnet_delete = {}
 
 
 class PluginTest(unittest.TestCase):
@@ -305,15 +321,23 @@ class PluginTest(unittest.TestCase):
         def ip_to_digit(ip):
             return int(ip[ip.rindex('.') + 1:])
 
-        pool_range = list(range(ip_to_digit(t_gateway_ip),
-                                ip_to_digit(pool['end']) + 1))
-        # we include the top gateway ip in the bottom ip allocation pool
-        b_pool_range1 = list(range(ip_to_digit(b_pools[0]['start']),
-                                   ip_to_digit(b_pools[0]['end']) + 1))
-        b_pool_range2 = list(range(ip_to_digit(b_pools[1]['start']),
-                                   ip_to_digit(b_pools[1]['end']) + 1))
-        b_pool_range = b_pool_range1 + [
-            ip_to_digit(b_gateway_ip)] + b_pool_range2
+        if t_gateway_ip:
+            pool_range = list(range(ip_to_digit(t_gateway_ip),
+                                    ip_to_digit(pool['end']) + 1))
+            # we include the top gateway ip in the bottom ip allocation pool
+            b_pool_range1 = list(range(ip_to_digit(b_pools[0]['start']),
+                                       ip_to_digit(b_pools[0]['end']) + 1))
+            b_pool_range2 = list(range(ip_to_digit(b_pools[1]['start']),
+                                       ip_to_digit(b_pools[1]['end']) + 1))
+            b_pool_range = b_pool_range1 + [
+                ip_to_digit(b_gateway_ip)] + b_pool_range2
+        else:
+            self.assertIsNone(t_gateway_ip)
+            self.assertIsNone(b_gateway_ip)
+            pool_range = list(range(ip_to_digit(pool['start']),
+                                    ip_to_digit(pool['end'])))
+            b_pool_range = list(range(ip_to_digit(b_pools[0]['start']),
+                                      ip_to_digit(b_pools[0]['end'])))
         port.pop('name')
         b_port.pop('name')
         self.assertDictEqual(net, b_net)
@@ -344,6 +368,12 @@ class PluginTest(unittest.TestCase):
         t_net, t_subnet, t_port, _ = self._prepare_resource()
         self._validate(t_net, t_subnet, t_port)
 
+    @patch.object(t_context, 'get_context_from_neutron_context', new=mock.Mock)
+    def test_get_network_no_gateway(self):
+        t_net, t_subnet, t_port, _ = self._prepare_resource()
+        update_resource('subnet', True, t_subnet['id'], {'gateway_ip': None})
+        self._validate(t_net, t_subnet, t_port)
+
     @patch.object(t_context, 'get_context_from_neutron_context', new=mock.Mock)
     @patch.object(client.Client, 'get_admin_token', new=mock.Mock)
     def test_get_networks(self):
@@ -468,6 +498,7 @@ class PluginTest(unittest.TestCase):
         # "agent" extension and body contains tunnel ip
         mock_agent.assert_has_calls([mock.call(self.context, agent_state)])
 
+    @patch.object(FakePlugin, '_ensure_trunk', new=mock.Mock)
     @patch.object(t_context, 'get_context_from_neutron_context', new=mock.Mock)
     def test_get_port(self):
         t_net, t_subnet, t_port, _ = self._prepare_resource()
@@ -477,6 +508,35 @@ class PluginTest(unittest.TestCase):
         b_port = get_resource('port', False, t_port['id'])
         self.assertDictEqual(t_port, b_port)
 
+    @patch.object(t_context, 'get_context_from_neutron_context', new=mock.Mock)
+    @patch.object(plugin.TricirclePlugin, '_handle_security_group',
+                  new=mock.Mock)
+    @patch.object(directory, 'get_plugin', new=fake_get_trunk_plugin)
+    @patch.object(FakeTrunkPlugin, 'create_trunk')
+    def test_get_port_trunk(self, mock_create_trunk):
+        _, _, parent_port, _ = self._prepare_resource()
+        _, _, subport, _ = self._prepare_resource()
+        t_trunk_id = uuidutils.generate_uuid()
+        parent_port['trunk_details'] = {'trunk_id': t_trunk_id,
+                                        'sub_ports': [
+                                            {"segmentation_type": "vlan",
+                                             "port_id": subport['id'],
+                                             "segmentation_id": 100}]}
+        t_trunk = {
+            'id': t_trunk_id,
+            'name': 'top_trunk_1',
+            'status': 'DOWN',
+            'description': 'created',
+            'admin_state_up': True,
+            'port_id': parent_port['id'],
+            'sub_ports': []
+        }
+        TOP_TRUNKS.append(t_trunk)
+
+        self.plugin.get_port(self.context, parent_port['id'])
+        mock_create_trunk.assert_called_once_with(self.context,
+                                                  {'trunk': t_trunk})
+
     @patch.object(t_context, 'get_context_from_neutron_context', new=mock.Mock)
     def test_get_ports(self):
         t_net, t_subnet, t_port, t_sg = self._prepare_resource()
@@ -570,7 +630,7 @@ class PluginTest(unittest.TestCase):
                                           'host': 'fake_host',
                                           'device': 'compute:None'}}})
 
-        cfg.CONF.set_override('l2gw_tunnel_ip', '', 'tricircle')
+        cfg.CONF.set_override('l2gw_tunnel_ip', None, 'tricircle')
         cfg.CONF.set_override('cross_pod_vxlan_mode', 'l2gw', 'client')
         self.plugin.update_port(self.context, port_id, update_body)
         # l2gw mode, but l2 gateway tunnel ip is not configured
@@ -618,5 +678,4 @@ class PluginTest(unittest.TestCase):
         self.assertEqual(b_port['device_owner'], 'network:dhcp')
 
     def tearDown(self):
-        for res in RES_LIST:
-            del res[:]
+        test_utils.get_resource_store().clean()
