@@ -367,12 +367,15 @@ class XManagerTest(unittest.TestCase):
     @patch.object(FakeClient, 'update_routers')
     def test_configure_extra_routes_with_floating_ips(self, mock_update):
         top_router_id = 'router_id'
+        project_id = uuidutils.generate_uuid()
         bridge_infos = self._prepare_east_west_network_test(top_router_id)
         ns_bridge_ip, ns_router_id = self._prepare_snat_test(top_router_id)
         self._prepare_dnat_test()
-        db_api.new_job(self.context, constants.JT_ROUTER, top_router_id)
-        self.xmanager.configure_extra_routes(
-            self.context, payload={constants.JT_ROUTER: top_router_id})
+        db_api.new_job(self.context, project_id, constants.JT_CONFIGURE_ROUTE,
+                       top_router_id)
+        self.xmanager.configure_route(
+            self.context,
+            payload={constants.JT_CONFIGURE_ROUTE: top_router_id})
         calls = []
         ns_routes = []
         for i in range(2):
@@ -394,11 +397,14 @@ class XManagerTest(unittest.TestCase):
     @patch.object(FakeClient, 'update_routers')
     def test_configure_extra_routes_with_external_network(self, mock_update):
         top_router_id = 'router_id'
+        project_id = uuidutils.generate_uuid()
         bridge_infos = self._prepare_east_west_network_test(top_router_id)
         ns_bridge_ip, ns_router_id = self._prepare_snat_test(top_router_id)
-        db_api.new_job(self.context, constants.JT_ROUTER, top_router_id)
-        self.xmanager.configure_extra_routes(
-            self.context, payload={constants.JT_ROUTER: top_router_id})
+        db_api.new_job(self.context, project_id, constants.JT_CONFIGURE_ROUTE,
+                       top_router_id)
+        self.xmanager.configure_route(
+            self.context,
+            payload={constants.JT_CONFIGURE_ROUTE: top_router_id})
         calls = []
         ns_routes = []
         for i in range(2):
@@ -418,12 +424,15 @@ class XManagerTest(unittest.TestCase):
         self._check_extra_routes_calls(calls, mock_update.call_args_list)
 
     @patch.object(FakeClient, 'update_routers')
-    def test_configure_extra_routes(self, mock_update):
+    def test_configure_route(self, mock_update):
         top_router_id = 'router_id'
+        project_id = uuidutils.generate_uuid()
         bridge_infos = self._prepare_east_west_network_test(top_router_id)
-        db_api.new_job(self.context, constants.JT_ROUTER, top_router_id)
-        self.xmanager.configure_extra_routes(
-            self.context, payload={constants.JT_ROUTER: top_router_id})
+        db_api.new_job(self.context, project_id, constants.JT_CONFIGURE_ROUTE,
+                       top_router_id)
+        self.xmanager.configure_route(
+            self.context,
+            payload={constants.JT_CONFIGURE_ROUTE: top_router_id})
         calls = []
         for i in range(2):
             routes = []
@@ -538,9 +547,12 @@ class XManagerTest(unittest.TestCase):
         #       net3 is attached to R3
 
         target_router_id = 'top_router_3_id'
-        db_api.new_job(self.context, constants.JT_ROUTER, target_router_id)
-        self.xmanager.configure_extra_routes(
-            self.context, payload={constants.JT_ROUTER: target_router_id})
+        project_id = uuidutils.generate_uuid()
+        db_api.new_job(self.context, project_id,
+                       constants.JT_CONFIGURE_ROUTE, target_router_id)
+        self.xmanager.configure_route(
+            self.context,
+            payload={constants.JT_CONFIGURE_ROUTE: target_router_id})
 
         # for the following paths, packets will go to R3 via the interface
         # which is attached to R3
@@ -653,7 +665,8 @@ class XManagerTest(unittest.TestCase):
         RES_MAP['top']['subnet'].append(subnet_ipv6)
         RES_MAP['pod_1']['security_group'].append(sg)
 
-        db_api.new_job(self.context, constants.JT_SEG_RULE_SETUP, project_id)
+        db_api.new_job(self.context, project_id, constants.JT_SEG_RULE_SETUP,
+                       project_id)
         self.xmanager.configure_security_group_rules(
             self.context, payload={constants.JT_SEG_RULE_SETUP: project_id})
 
@@ -685,6 +698,94 @@ class XManagerTest(unittest.TestCase):
                                 'security_group_id': sg_id}]})]
         mock_create.assert_has_calls(calls)
 
+    @patch.object(FakeClient, 'delete_security_group_rules')
+    @patch.object(FakeClient, 'create_security_group_rules')
+    def test_configure_security_group_rules_duplicated_cidr(self, mock_create,
+                                                            mock_delete):
+        project_id = uuidutils.generate_uuid()
+        sg_id = uuidutils.generate_uuid()
+        sg_rule_id_1 = uuidutils.generate_uuid()
+        sg_rule_id_2 = uuidutils.generate_uuid()
+
+        sg = {'id': sg_id,
+              'tenant_id': project_id,
+              'name': 'default',
+              'security_group_rules': [{
+                  'id': sg_rule_id_1,
+                  'remote_group_id': sg_id,
+                  'direction': 'ingress',
+                  'remote_ip_prefix': None,
+                  'protocol': None,
+                  'ethertype': 'IPv4',
+                  'port_range_max': -1,
+                  'port_range_min': -1,
+                  'security_group_id': sg_id},
+                  {'id': sg_rule_id_2,
+                   'remote_group_id': None,
+                   'direction': 'engress',
+                   'remote_ip_prefix': None,
+                   'protocol': None,
+                   'ethertype': 'IPv4',
+                   'port_range_max': -1,
+                   'port_range_min': -1,
+                   'security_group_id': sg_id}]}
+        RES_MAP['top']['security_group'].append(sg)
+
+        for i in xrange(1, 3):
+            pod_dict = {'pod_id': 'pod_id_%d' % i,
+                        'region_name': 'pod_%d' % i,
+                        'az_name': 'az_name_%d' % i}
+            db_api.create_pod(self.context, pod_dict)
+
+            network = {'id': 'network_%d_id' % i,
+                       'tenant_id': project_id}
+            # we create two subnets with identical cidr but different
+            # allocation pools
+            subnet = {'id': 'subnet_%d_id' % i,
+                      'network_id': network['id'],
+                      'cidr': '10.0.1.0/24',
+                      'gateway_ip': '10.0.1.%d' % i,
+                      'tenant_id': project_id,
+                      'allocation_pools': {'start': '10.0.1.%d' % 10 * i,
+                                           'end': '10.0.1.%d' % (10 * i + 9)},
+                      'ip_version': q_constants.IP_VERSION_4}
+            RES_MAP['top']['network'].append(network)
+            RES_MAP['top']['subnet'].append(subnet)
+
+            region_name = 'pod_%d' % i
+            RES_MAP[region_name]['security_group'].append(sg)
+            route = {'top_id': sg_id, 'bottom_id': sg_id,
+                     'pod_id': pod_dict['pod_id'],
+                     'resource_type': 'security_group'}
+            with self.context.session.begin():
+                core.create_resource(self.context, models.ResourceRouting,
+                                     route)
+
+        db_api.new_job(self.context, project_id, constants.JT_SEG_RULE_SETUP,
+                       project_id)
+        self.xmanager.configure_security_group_rules(
+            self.context, payload={constants.JT_SEG_RULE_SETUP: project_id})
+
+        calls = [mock.call(self.context, sg_rule_id_1)]
+        mock_delete.assert_has_calls(calls)
+        call_rules_id = [
+            call_arg[0][1] for call_arg in mock_delete.call_args_list]
+        # bottom security group already has sg_rule_id_2, so this rule will
+        # not be deleted
+        self.assertNotIn(sg_rule_id_2, call_rules_id)
+
+        calls = [mock.call(self.context,
+                           {'security_group_rules': [
+                               {'remote_group_id': None,
+                                'direction': 'ingress',
+                                'remote_ip_prefix': '10.0.1.0/24',
+                                'protocol': None,
+                                'ethertype': 'IPv4',
+                                'port_range_max': -1,
+                                'port_range_min': -1,
+                                'security_group_id': sg_id}]})]
+        mock_create.assert_has_calls(calls)
+
     @patch.object(helper.NetworkHelper, '_get_client', new=fake_get_client)
     @patch.object(FakeXJobAPI, 'setup_shadow_ports')
     def test_setup_shadow_ports(self, mock_setup):
@@ -727,8 +828,8 @@ class XManagerTest(unittest.TestCase):
             '192.168.1.102')
 
         resource_id = 'pod_id_1#' + net1_id
-        db_api.new_job(self.context, constants.JT_SHADOW_PORT_SETUP,
-                       resource_id)
+        db_api.new_job(self.context, project_id,
+                       constants.JT_SHADOW_PORT_SETUP, resource_id)
         self.xmanager.setup_shadow_ports(
             self.context,
             payload={constants.JT_SHADOW_PORT_SETUP: resource_id})
@@ -745,7 +846,8 @@ class XManagerTest(unittest.TestCase):
                       sd_ports[0]['binding:profile'])
 
         # check job to setup shadow ports for pod2 is registered
-        mock_setup.assert_called_once_with(self.context, 'pod_id_2', net1_id)
+        mock_setup.assert_called_once_with(self.context, project_id,
+                                           'pod_id_2', net1_id)
 
         # update shadow port to down and test again, this is possible when we
         # succeed to create shadow port but fail to update it to active
@@ -755,8 +857,8 @@ class XManagerTest(unittest.TestCase):
                              {'port': {'status': q_constants.PORT_STATUS_DOWN,
                                        'binding:profile': profile}})
 
-        db_api.new_job(self.context, constants.JT_SHADOW_PORT_SETUP,
-                       resource_id)
+        db_api.new_job(self.context, project_id,
+                       constants.JT_SHADOW_PORT_SETUP, resource_id)
         self.xmanager.setup_shadow_ports(
             self.context,
             payload={constants.JT_SHADOW_PORT_SETUP: resource_id})
@@ -767,8 +869,8 @@ class XManagerTest(unittest.TestCase):
 
         # manually trigger shadow ports setup in pod2
         resource_id = 'pod_id_2#' + net1_id
-        db_api.new_job(self.context, constants.JT_SHADOW_PORT_SETUP,
-                       resource_id)
+        db_api.new_job(self.context, project_id,
+                       constants.JT_SHADOW_PORT_SETUP, resource_id)
         self.xmanager.setup_shadow_ports(
             self.context,
             payload={constants.JT_SHADOW_PORT_SETUP: resource_id})
@@ -789,8 +891,9 @@ class XManagerTest(unittest.TestCase):
             pass
 
         fake_id = 'fake_id'
+        fake_project_id = uuidutils.generate_uuid()
         payload = {job_type: fake_id}
-        db_api.new_job(self.context, job_type, fake_id)
+        db_api.new_job(self.context, fake_project_id, job_type, fake_id)
         fake_handle(None, self.context, payload=payload)
 
         logs = core.query_resource(self.context, models.AsyncJobLog, [], [])
@@ -806,8 +909,9 @@ class XManagerTest(unittest.TestCase):
             raise Exception()
 
         fake_id = 'fake_id'
+        fake_project_id = uuidutils.generate_uuid()
         payload = {job_type: fake_id}
-        db_api.new_job(self.context, job_type, fake_id)
+        db_api.new_job(self.context, fake_project_id, job_type, fake_id)
         fake_handle(None, self.context, payload=payload)
 
         jobs = core.query_resource(self.context, models.AsyncJob, [], [])
@@ -828,8 +932,9 @@ class XManagerTest(unittest.TestCase):
             pass
 
         fake_id = uuidutils.generate_uuid()
+        fake_project_id = uuidutils.generate_uuid()
         payload = {job_type: fake_id}
-        db_api.new_job(self.context, job_type, fake_id)
+        db_api.new_job(self.context, fake_project_id, job_type, fake_id)
         expired_job = {
             'id': uuidutils.generate_uuid(),
             'type': job_type,
@@ -860,8 +965,9 @@ class XManagerTest(unittest.TestCase):
         mock_get.return_value = None
 
         fake_id = uuidutils.generate_uuid()
+        fake_project_id = uuidutils.generate_uuid()
         payload = {job_type: fake_id}
-        db_api.new_job(self.context, job_type, fake_id)
+        db_api.new_job(self.context, fake_project_id, job_type, fake_id)
         fake_handle(None, self.context, payload=payload)
 
         # nothing to assert, what we test is that fake_handle can exit when
@@ -872,28 +978,28 @@ class XManagerTest(unittest.TestCase):
         mock_now.return_value = datetime.datetime(2000, 1, 2, 12, 0, 0)
         job_dict_list = [
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 0, 0),
-             'resource_id': 'uuid1', 'type': 'res1',
+             'resource_id': 'uuid1', 'type': 'res1', 'project_id': "uuid1",
              'status': constants.JS_Fail},  # job_uuid1
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 5, 0),
-             'resource_id': 'uuid1', 'type': 'res1',
+             'resource_id': 'uuid1', 'type': 'res1', 'project_id': "uuid1",
              'status': constants.JS_Fail},  # job_uuid3
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 20, 0),
-             'resource_id': 'uuid2', 'type': 'res2',
+             'resource_id': 'uuid2', 'type': 'res2', 'project_id': "uuid1",
              'status': constants.JS_Fail},  # job_uuid5
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 15, 0),
-             'resource_id': 'uuid2', 'type': 'res2',
+             'resource_id': 'uuid2', 'type': 'res2', 'project_id': "uuid1",
              'status': constants.JS_Fail},  # job_uuid7
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 25, 0),
-             'resource_id': 'uuid3', 'type': 'res3',
+             'resource_id': 'uuid3', 'type': 'res3', 'project_id': "uuid1",
              'status': constants.JS_Success},  # job_uuid9
             {'timestamp': datetime.datetime(2000, 1, 1, 12, 30, 0),
-             'resource_id': 'uuid4', 'type': 'res4',
+             'resource_id': 'uuid4', 'type': 'res4', 'project_id': "uuid1",
              'status': constants.JS_New},  # job_uuid11
             {'timestamp': datetime.datetime(1999, 12, 31, 12, 0, 0),
-             'resource_id': 'uuid5', 'type': 'res5',
+             'resource_id': 'uuid5', 'type': 'res5', 'project_id': "uuid1",
              'status': constants.JS_Fail},  # job_uuid13
             {'timestamp': datetime.datetime(1999, 12, 31, 11, 59, 59),
-             'resource_id': 'uuid6', 'type': 'res6',
+             'resource_id': 'uuid6', 'type': 'res6', 'project_id': "uuid1",
              'status': constants.JS_Fail}]  # job_uuid15
         for i, job_dict in enumerate(job_dict_list, 1):
             job_dict['id'] = 'job_uuid%d' % (2 * i - 1)
@@ -907,10 +1013,11 @@ class XManagerTest(unittest.TestCase):
         # for res3 + uuid3, the latest job's status is "Success", not returned
         # for res6 + uuid6, the latest job is out of the redo time span
         expected_failed_jobs = [
-            {'resource_id': 'uuid1', 'type': 'res1'},
-            {'resource_id': 'uuid2', 'type': 'res2'},
-            {'resource_id': 'uuid5', 'type': 'res5'}]
-        expected_new_jobs = [{'resource_id': 'uuid4', 'type': 'res4'}]
+            {'resource_id': 'uuid1', 'type': 'res1', 'project_id': "uuid1"},
+            {'resource_id': 'uuid2', 'type': 'res2', 'project_id': "uuid1"},
+            {'resource_id': 'uuid5', 'type': 'res5', 'project_id': "uuid1"}]
+        expected_new_jobs = [{'resource_id': 'uuid4', 'type': 'res4',
+                              'project_id': "uuid1"}]
         (failed_jobs,
          new_jobs) = db_api.get_latest_failed_or_new_jobs(self.context)
         six.assertCountEqual(self, expected_failed_jobs, failed_jobs)
