@@ -31,6 +31,7 @@ import neutronclient.common.exceptions as q_cli_exceptions
 
 from tricircle.common import client
 from tricircle.common import constants
+import tricircle.common.context as t_context
 from tricircle.common import xrpcapi
 import tricircle.db.api as db_api
 import tricircle.network.exceptions as t_network_exc
@@ -67,15 +68,16 @@ def _job_handle(job_type):
                 if delta.seconds >= CONF.worker_handle_timeout:
                     # quit when this handle is running for a long time
                     break
-                time_new = db_api.get_latest_timestamp(ctx, constants.JS_New,
-                                                       job_type, resource_id)
-                if not time_new:
+                job_new = db_api.get_latest_job(
+                    ctx, constants.JS_New, job_type, resource_id)
+                if not job_new:
                     break
-                time_success = db_api.get_latest_timestamp(
+                job_succ = db_api.get_latest_job(
                     ctx, constants.JS_Success, job_type, resource_id)
-                if time_success and time_success >= time_new:
+                if job_succ and job_succ['timestamp'] >= job_new['timestamp']:
                     break
-                job = db_api.register_job(ctx, job_type, resource_id)
+                job = db_api.register_job(ctx, job_new['project_id'], job_type,
+                                          resource_id)
                 if not job:
                     # fail to obtain the lock, let other worker handle the job
                     running_job = db_api.get_running_job(ctx, job_type,
@@ -94,7 +96,7 @@ def _job_handle(job_type):
                         # previous running job expires, we set its status to
                         # fail and try again to obtain the lock
                         db_api.finish_job(ctx, running_job['id'], False,
-                                          time_new)
+                                          job_new['timestamp'])
                         LOG.warning('Job %(job)s of type %(job_type)s for '
                                     'resource %(resource)s expires, set '
                                     'its state to Fail',
@@ -111,14 +113,15 @@ def _job_handle(job_type):
                 try:
                     func(*args, **kwargs)
                 except Exception:
-                    db_api.finish_job(ctx, job['id'], False, time_new)
+                    db_api.finish_job(ctx, job['id'], False,
+                                      job_new['timestamp'])
                     LOG.error('Job %(job)s of type %(job_type)s for '
                               'resource %(resource)s fails',
                               {'job': job['id'],
                                'job_type': job_type,
                                'resource': resource_id})
                     break
-                db_api.finish_job(ctx, job['id'], True, time_new)
+                db_api.finish_job(ctx, job['id'], True, job_new['timestamp'])
                 eventlet.sleep(CONF.worker_sleep_time)
         return handle_args
     return handle_func
@@ -145,13 +148,14 @@ class XManager(PeriodicTasks):
         self.additional_endpoints = []
         self.clients = {constants.TOP: client.Client()}
         self.job_handles = {
-            constants.JT_ROUTER: self.configure_extra_routes,
+            constants.JT_CONFIGURE_ROUTE: self.configure_route,
             constants.JT_ROUTER_SETUP: self.setup_bottom_router,
             constants.JT_PORT_DELETE: self.delete_server_port,
             constants.JT_SEG_RULE_SETUP: self.configure_security_group_rules,
             constants.JT_NETWORK_UPDATE: self.update_network,
             constants.JT_SUBNET_UPDATE: self.update_subnet,
-            constants.JT_SHADOW_PORT_SETUP: self.setup_shadow_ports}
+            constants.JT_SHADOW_PORT_SETUP: self.setup_shadow_ports,
+            constants.JT_TRUNK_SYNC: self.sync_trunk}
         self.helper = helper.NetworkHelper()
         self.xjob_handler = xrpcapi.XJobAPI()
         super(XManager, self).__init__()
@@ -251,13 +255,16 @@ class XManager(PeriodicTasks):
         job_index = random.randint(0, len(jobs) - 1)
         job_type = jobs[job_index]['type']
         resource_id = jobs[job_index]['resource_id']
+        project_id = jobs[job_index]['project_id']
         payload = {job_type: resource_id}
         LOG.debug('Redo %(status)s job for %(resource_id)s of type '
                   '%(job_type)s',
                   {'status': 'new' if is_new_job else 'failed',
                    'resource_id': resource_id, 'job_type': job_type})
+        # this is an admin context, we set the correct project id
+        ctx.tenant = project_id
         if not is_new_job:
-            db_api.new_job(ctx, job_type, resource_id)
+            db_api.new_job(ctx, project_id, job_type, resource_id)
         self.job_handles[job_type](ctx, payload=payload)
 
     @staticmethod
@@ -512,6 +519,15 @@ class XManager(PeriodicTasks):
         (b_pod_id,
          t_router_id, t_net_id) = payload[constants.JT_ROUTER_SETUP].split('#')
 
+        t_client = self._get_client()
+        t_pod = db_api.get_top_pod(ctx)
+        t_router = t_client.get_routers(ctx, t_router_id)
+
+        if not t_router:
+            # we just end this job if top router no longer exists
+            return
+
+        project_id = t_router['tenant_id']
         if b_pod_id == constants.POD_NOT_SPECIFIED:
             mappings = db_api.get_bottom_mappings_by_top_id(
                 ctx, t_net_id, constants.RT_NETWORK)
@@ -520,15 +536,9 @@ class XManager(PeriodicTasks):
                 # NOTE(zhiyuan) we create one job for each pod to avoid
                 # conflict caused by different workers operating the same pod
                 self.xjob_handler.setup_bottom_router(
-                    ctx, t_net_id, t_router_id, b_pod['pod_id'])
+                    ctx, project_id, t_net_id, t_router_id, b_pod['pod_id'])
             return
 
-        t_client = self._get_client()
-        t_pod = db_api.get_top_pod(ctx)
-        t_router = t_client.get_routers(ctx, t_router_id)
-        if not t_router:
-            # we just end this job if top router no longer exists
-            return
         t_net = t_client.get_networks(ctx, t_net_id)
         if not t_net:
             # we just end this job if top network no longer exists
@@ -566,11 +576,12 @@ class XManager(PeriodicTasks):
             ctx, t_pod, b_pod, t_client, t_net, t_router, t_bridge_net,
             t_bridge_subnet, is_ext_net_pod)
         if not is_local_router:
-            self.xjob_handler.configure_extra_routes(ctx, t_router_id)
+            self.xjob_handler.configure_route(ctx, project_id,
+                                              t_router_id)
 
-    @_job_handle(constants.JT_ROUTER)
-    def configure_extra_routes(self, ctx, payload):
-        t_router_id = payload[constants.JT_ROUTER]
+    @_job_handle(constants.JT_CONFIGURE_ROUTE)
+    def configure_route(self, ctx, payload):
+        t_router_id = payload[constants.JT_CONFIGURE_ROUTE]
         t_client = self._get_client()
         t_router = t_client.get_routers(ctx, t_router_id)
         if not t_router:
@@ -808,6 +819,7 @@ class XManager(PeriodicTasks):
                     ctx, [{'key': 'tenant_id', 'comparator': 'eq',
                            'value': project_id}])
                 bridge_ip_net = netaddr.IPNetwork(CONF.client.bridge_cidr)
+                subnet_cidr_set = set()
                 for subnet in subnets:
                     ip_net = netaddr.IPNetwork(subnet['cidr'])
                     if ip_net in bridge_ip_net:
@@ -816,6 +828,9 @@ class XManager(PeriodicTasks):
                     # Tricircle has not supported IPv6 well yet,
                     # so we ignore seg rules temporarily.
                     if subnet['ip_version'] == q_constants.IP_VERSION_4:
+                        if subnet['cidr'] in subnet_cidr_set:
+                            continue
+                        subnet_cidr_set.add(subnet['cidr'])
                         new_b_rules.append(
                             self._construct_bottom_rule(t_rule, '',
                                                         subnet['cidr']))
@@ -867,19 +882,22 @@ class XManager(PeriodicTasks):
         """
         (b_pod_id, t_network_id) = payload[
             constants.JT_NETWORK_UPDATE].split('#')
-        if b_pod_id == constants.POD_NOT_SPECIFIED:
-            mappings = db_api.get_bottom_mappings_by_top_id(
-                ctx, t_network_id, constants.RT_NETWORK)
-            b_pods = [mapping[0] for mapping in mappings]
-            for b_pod in b_pods:
-                self.xjob_handler.update_network(ctx, t_network_id,
-                                                 b_pod['pod_id'])
-            return
 
         t_client = self._get_client()
         t_network = t_client.get_networks(ctx, t_network_id)
         if not t_network:
             return
+
+        project_id = t_network['tenant_id']
+        if b_pod_id == constants.POD_NOT_SPECIFIED:
+            mappings = db_api.get_bottom_mappings_by_top_id(
+                ctx, t_network_id, constants.RT_NETWORK)
+            b_pods = [mapping[0] for mapping in mappings]
+            for b_pod in b_pods:
+                self.xjob_handler.update_network(ctx, project_id,
+                                                 t_network_id, b_pod['pod_id'])
+            return
+
         b_pod = db_api.get_pod(ctx, b_pod_id)
         b_region_name = b_pod['region_name']
         b_client = self._get_client(region_name=b_region_name)
@@ -918,19 +936,22 @@ class XManager(PeriodicTasks):
         """
         (b_pod_id, t_subnet_id) = payload[
             constants.JT_SUBNET_UPDATE].split('#')
-        if b_pod_id == constants.POD_NOT_SPECIFIED:
-            mappings = db_api.get_bottom_mappings_by_top_id(
-                ctx, t_subnet_id, constants.RT_SUBNET)
-            b_pods = [mapping[0] for mapping in mappings]
-            for b_pod in b_pods:
-                self.xjob_handler.update_subnet(ctx, t_subnet_id,
-                                                b_pod['pod_id'])
-            return
 
         t_client = self._get_client()
         t_subnet = t_client.get_subnets(ctx, t_subnet_id)
         if not t_subnet:
             return
+
+        project_id = t_subnet['tenant_id']
+        if b_pod_id == constants.POD_NOT_SPECIFIED:
+            mappings = db_api.get_bottom_mappings_by_top_id(
+                ctx, t_subnet_id, constants.RT_SUBNET)
+            b_pods = [mapping[0] for mapping in mappings]
+            for b_pod in b_pods:
+                self.xjob_handler.update_subnet(ctx, project_id,
+                                                t_subnet_id, b_pod['pod_id'])
+            return
+
         b_pod = db_api.get_pod(ctx, b_pod_id)
         b_region_name = b_pod['region_name']
         b_subnet_id = db_api.get_bottom_id_by_top_id_region_name(
@@ -1097,4 +1118,138 @@ class XManager(PeriodicTasks):
                 ctx, sw_port_id, update_body)
 
         for pod_id in sync_pod_list:
-            self.xjob_handler.setup_shadow_ports(ctx, pod_id, t_net_id)
+            self.xjob_handler.setup_shadow_ports(ctx, project_id,
+                                                 pod_id, t_net_id)
+
+    def _get_bottom_need_created_subports(self, ctx, t_ctx, project_id,
+                                          trunk_id, add_subport_ids):
+        t_client = self._get_client()
+        need_created_ports = []
+        port_filters = [{'key': 'device_id',
+                        'comparator': 'eq',
+                         'value': trunk_id},
+                        {'key': 'device_owner',
+                         'comparator': 'eq',
+                         'value': constants.DEVICE_OWNER_SUBPORT}
+                        ]
+        trunk_subports = t_client.list_ports(ctx, filters=port_filters)
+        map_filters = [{'key': 'resource_type',
+                        'comparator': 'eq',
+                        'value': constants.RT_PORT},
+                       {'key': 'project_id',
+                        'comparator': 'eq',
+                        'value': project_id}]
+
+        port_mappings = db_api.list_resource_routings(t_ctx, map_filters)
+        mapping_port_ids = [port['top_id'] for port in port_mappings]
+        pop_attrs = ['status', 'tags', 'updated_at',
+                     'created_at', 'revision_number', 'id']
+        for port in trunk_subports:
+            if (port['id'] in add_subport_ids
+                    and port['id'] not in mapping_port_ids):
+                port['device_id'] = port['id']
+                # pop attributes which not allowed in POST
+                for attr in pop_attrs:
+                    port.pop(attr, None)
+                need_created_ports.append(port)
+
+        return need_created_ports
+
+    def _create_trunk_subport_mapping(self, t_ctx, project_id, pod, ports):
+        entries = []
+        for port in ports:
+            port['id'] = port['device_id']
+            entries.extend(self.helper.extract_resource_routing_entries(port))
+        self.helper.ensure_resource_mapping(t_ctx, project_id, pod, entries)
+
+    def _create_bottom_trunk_subports(self, ctx, target_pod,
+                                      full_create_bodys, max_bulk_size):
+        cursor = 0
+        ret_port_ids = []
+        b_client = self._get_client(target_pod['region_name'])
+        while cursor < len(full_create_bodys):
+            ret_port_ids.extend(self.helper.prepare_ports_with_retry(
+                ctx, b_client,
+                full_create_bodys[cursor: cursor + max_bulk_size]))
+            cursor += max_bulk_size
+        return ret_port_ids
+
+    @_job_handle(constants.JT_TRUNK_SYNC)
+    def sync_trunk(self, ctx, payload):
+        b_pod_id, t_trunk_id = payload[constants.JT_TRUNK_SYNC].split('#')
+        b_pod = db_api.get_pod(ctx, b_pod_id)
+        b_region_name = b_pod['region_name']
+        b_client = self._get_client(region_name=b_region_name)
+        b_trunk_id = db_api.get_bottom_id_by_top_id_region_name(
+            ctx, t_trunk_id, b_region_name, constants.RT_TRUNK)
+        if not b_trunk_id:
+            return
+        t_client = self._get_client()
+        t_trunk = t_client.get_trunks(ctx, t_trunk_id)
+        # delete trunk action
+        if not t_trunk:
+            b_client.delete_trunks(ctx, b_trunk_id)
+            db_api.delete_mappings_by_top_id(ctx, t_trunk_id)
+            return
+
+        # update trunk action
+        b_trunk = b_client.get_trunks(ctx, b_trunk_id)
+
+        if not b_trunk:
+            LOG.error('trunk: %(trunk_id)s not found, pod name: %(name)s',
+                      {'trunk_id': b_trunk_id, 'name': b_region_name})
+            return
+
+        body = {
+            'trunk':
+                {'description': t_trunk['description'],
+                 'admin_state_up': t_trunk['admin_state_up']}
+        }
+
+        t_subports = set(
+            [(subport['port_id'],
+              subport['segmentation_id']) for subport in t_trunk['sub_ports']])
+        b_subports = set(
+            [(subport['port_id'],
+              subport['segmentation_id']) for subport in b_trunk['sub_ports']])
+        add_subports = t_subports - b_subports
+        del_subports = b_subports - t_subports
+
+        add_subport_bodies = [
+            {'port_id': subport[0],
+             'segmentation_type': 'vlan',
+             'segmentation_id': subport[1]} for subport in add_subports]
+
+        del_subport_bodies = [
+            {'port_id': subport[0]} for subport in del_subports]
+
+        try:
+            b_client.update_trunks(ctx, b_trunk_id, body)
+            # must first delete subports, then add subports, otherwise it
+            # will lead to the addition of existing subports
+            if del_subport_bodies:
+                b_client.action_trunks(ctx, 'remove_subports', b_trunk_id,
+                                       {'sub_ports': del_subport_bodies})
+
+            # create bottom ports bulk
+            if add_subport_bodies:
+                project_id = t_trunk['project_id']
+                t_ctx = t_context.get_context_from_neutron_context(ctx)
+                max_bulk_size = CONF.client.max_trunk_subports_bulk_size
+                add_subport_ids = [
+                    subport['port_id'] for subport in add_subport_bodies]
+                need_created_ports = self._get_bottom_need_created_subports(
+                    ctx, t_ctx, project_id, t_trunk_id, add_subport_ids)
+                if need_created_ports:
+                    self._create_bottom_trunk_subports(
+                        ctx, b_pod, need_created_ports, max_bulk_size)
+                    self._create_trunk_subport_mapping(
+                        ctx, project_id, b_pod, need_created_ports)
+                    self.xjob_handler.configure_security_group_rules(
+                        t_ctx, project_id)
+
+                b_client.action_trunks(ctx, 'add_subports', b_trunk_id,
+                                       {'sub_ports': add_subport_bodies})
+        except q_cli_exceptions.NotFound:
+            LOG.error('trunk: %(trunk_id)s not found, pod name: %(name)s',
+                      {'trunk_id': b_trunk_id, 'name': b_region_name})
